@@ -29,8 +29,8 @@ tools = [
 ['climate_tools', 3, 'Annual Course of Daily Insolation'],
 ['climate_tools', 4, 'Daily Insolation over Latitude'],
 ['climate_tools', 5, 'Monthly Global by Latitude'],
-['climate_tools', 6, 'ETpot (after Hargreaves, Table)'],
-['climate_tools', 7, 'Daily to Hourly ETpot'],
+['climate_tools', 6, 'Evapotranspiration (Table)'],
+['climate_tools', 7, 'Daily to Hourly Evapotranspiration'],
 ['contrib_perego', 0, 'Average With Thereshold 1'],
 ['contrib_perego', 1, 'Average With Thereshold 2'],
 ['contrib_perego', 2, 'Average With Thereshold 3'],
@@ -106,7 +106,7 @@ tools = [
 ['grid_analysis', 8, 'Pattern Analysis'],
 ['grid_analysis', 9, 'Layer of extreme value'],
 ['grid_analysis', 10, 'Analytical Hierarchy Process'],
-['grid_analysis', 11, 'Ordered Weighted Averaging (OWA)'],
+['grid_analysis', 11, 'Ordered Weighted Averaging'],
 ['grid_analysis', 12, 'Aggregation Index'],
 ['grid_analysis', 13, 'Cross-Classification and Tabulation'],
 ['grid_analysis', 14, 'Soil Texture Classification'],
@@ -164,10 +164,10 @@ tools = [
 ['grid_spline', 1, 'Thin Plate Spline'],
 ['grid_spline', 2, 'Thin Plate Spline (TIN)'],
 ['grid_spline', 3, 'B-Spline Approximation'],
-['grid_spline', 4, 'Multilevel B-Spline Interpolation'],
-['grid_spline', 5, 'Multilevel B-Spline Interpolation (from Grid)'],
+['grid_spline', 4, 'Multilevel B-Spline'],
+['grid_spline', 5, 'Multilevel B-Spline from Grid Points'],
 ['grid_spline', 6, 'Cubic Spline Approximation'],
-['grid_spline', 7, 'Multilevel B-Spline Interpolation for Categories'],
+['grid_spline', 7, 'Multilevel B-Spline for Categories'],
 ['grid_tools', 0, 'Resampling'],
 ['grid_tools', 1, 'Aggregate'],
 ['grid_tools', 3, 'Mosaicking'],
@@ -210,8 +210,8 @@ tools = [
 ['imagery_classification', 1, 'K-Means Clustering for Grids'],
 ['imagery_classification', 2, 'Confusion Matrix (Two Grids)'],
 ['imagery_classification', 3, 'Decision Tree'],
-['imagery_classification', 4, 'Supervised Classification for Shapes'],
-['imagery_classification', 5, 'Supervised Classification for Tables'],
+['imagery_classification', 4, 'Supervised Classification (Shapes)'],
+['imagery_classification', 5, 'Supervised Classification (Tables)'],
 ['imagery_opencv', 0, 'Morphological Filter (OpenCV)'],
 ['imagery_opencv', 1, 'Fourier Transformation (OpenCV)'],
 ['imagery_opencv', 2, 'Single Value Decomposition (OpenCV)'],
@@ -378,7 +378,7 @@ tools = [
 ['shapes_points', 0, 'Convert Table to Points'],
 ['shapes_points', 1, 'Count Points in Polygons'],
 ['shapes_points', 2, 'Create Point Grid'],
-['shapes_points', 3, 'Point Distances'],
+['shapes_points', 3, 'Point to Point Distances'],
 ['shapes_points', 4, 'Populate Polygons with Points'],
 ['shapes_points', 5, 'Convert Lines to Points'],
 ['shapes_points', 6, 'Add Coordinates to Points'],
@@ -449,8 +449,8 @@ tools = [
 ['sim_erosion', 0, 'MMF-SAGA Soil Erosion Model'],
 ['sim_fire_spreading', 0, 'Fire Risk Analysis'],
 ['sim_fire_spreading', 1, 'Simulation'],
-['sim_hydrology', 0, 'Soil Moisture Content'],
-['sim_hydrology', 1, 'Kinematic Wave Overland Flow'],
+['sim_hydrology', 0, 'Top Soil Water Content'],
+['sim_hydrology', 1, 'Overland Flow (Kinematic Wave)'],
 ['sim_hydrology', 2, 'TOPMODEL'],
 ['sim_hydrology', 3, 'Water Retention Capacity'],
 ['sim_hydrology', 4, 'Diffuse Pollution Risk'],
@@ -460,7 +460,7 @@ tools = [
 ['sim_ihacres', 3, 'IHACRES Elevation Bands'],
 ['sim_ihacres', 4, 'IHACRES Elevation Bands Calibration'],
 ['statistics_grid', 0, 'Fast Representativeness'],
-['statistics_grid', 1, 'Residual Analysis (Grid)'],
+['statistics_grid', 1, 'Focal Statistics'],
 ['statistics_grid', 2, 'Representativeness (Grid)'],
 ['statistics_grid', 3, 'Radius of Variance (Grid)'],
 ['statistics_grid', 4, 'Statistics for Grids'],
@@ -490,12 +490,12 @@ tools = [
 ['statistics_regression', 4, 'GWR for Single Predictor Grid'],
 ['statistics_regression', 5, 'GWR for Multiple Predictors (Gridded Model Output)'],
 ['statistics_regression', 6, 'GWR for Multiple Predictor Grids'],
-['statistics_regression', 7, 'GWR for Multiple Predictors'],
+['statistics_regression', 7, 'GWR for Multiple Predictors (Shapes)'],
 ['statistics_regression', 8, 'Multiple Regression Analysis (Grid and Predictor Grids)'],
-['statistics_regression', 9, 'Polynomial Trend from Grids'],
+['statistics_regression', 9, 'Cellwise Trend for Grids'],
 ['statistics_regression', 10, 'Trend Analysis'],
 ['statistics_regression', 11, 'Trend Analysis (Shapes)'],
-['statistics_regression', 12, 'Multiple Linear Regression Analysis'],
+['statistics_regression', 12, 'Multiple Linear Regression Analysis (Table)'],
 ['statistics_regression', 13, 'Multiple Linear Regression Analysis (Shapes)'],
 ['statistics_regression', 14, 'GWR for Grid Downscaling'],
 ['statistics_regression', 15, 'Zonal Multiple Regression Analysis (Points and Predictor Grids)'],
@@ -546,7 +546,7 @@ tools = [
 ['ta_morphometry', 10, 'Mass Balance Index'],
 ['ta_morphometry', 11, 'Effective Air Flow Heights'],
 ['ta_morphometry', 12, 'Diurnal Anisotropic Heat'],
-['ta_morphometry', 13, 'Land Surface Temperature'],
+['ta_morphometry', 13, 'Land Surface Temperature (Lapse Rates)'],
 ['ta_morphometry', 14, 'Relative Heights and Slope Positions'],
 ['ta_morphometry', 15, 'Wind Effect (Windward / Leeward Index)'],
 ['ta_morphometry', 16, 'Terrain Ruggedness Index (TRI)'],
@@ -618,7 +618,7 @@ tools = [
 ['sim_rivflow', 3, 'RiverGridGeneration'],
 ['sim_rivflow', 4, 'GridCombination'],
 ['io_grid', 16, 'Import, Clip and Resample Grids'],
-['climate_tools', 8, 'ETpot (after Hargreaves, Grid)'],
+['climate_tools', 8, 'Evapotranspiration (Grid)'],
 ['grid_visualisation', 10, 'Select Look-up Table for Grid Visualization'],
 ['grid_tools', 32, 'Select Grid from List'],
 ['imagery_maxent', 0, 'Maximum Entropy Classifcation'],
@@ -715,7 +715,7 @@ tools = [
 ['grid_gridding', 10, 'Polygon Categories to Grid'],
 ['ta_lighting', 7, 'Potential Annual Insolation'],
 ['climate_tools', 19, 'Climate Classification'],
-['climate_tools', 20, 'Soil Water Balance'],
+['climate_tools', 20, 'Soil Water Balance (Annual)'],
 ['grid_tools', 39, 'Change Grid Values - Flood Fill'],
 ['io_gdal', 11, 'Import ASTER Scene'],
 ['shapes_tools', 29, 'Focal Mechanism (Beachball Plots)'],
@@ -732,5 +732,61 @@ tools = [
 ['table_tools', 23, 'Change Field Name'],
 ['ta_hydrology', 29, 'Flow Accumulation (Parallelizable)'],
 ['table_tools', 24, 'Formatted Text'],
-['table_tools', 25, 'Formatted Text [Shapes]']
+['table_tools', 25, 'Formatted Text [Shapes]'],
+['climate_tools', 21, 'PhenIps (Table)'],
+['climate_tools', 22, 'PhenIps (Grids, Annual)'],
+['grids_tools', 5, 'Nearest Neighbour (3D)'],
+['grids_tools', 6, 'Inverse Distance Weighted (3D)'],
+['grid_calculus', 21, 'Histogram Matching'],
+['grid_spline', 8, 'Multilevel B-Spline (3D)'],
+['grid_visualisation', 11, 'Create a Table from Look-up Table'],
+['imagery_opencv', 12, 'Logistic Regression (OpenCV)'],
+['imagery_segmentation', 4, 'Superpixel Segmentation'],
+['shapes_polygons', 23, 'Polygon Generalization'],
+['ta_lighting', 8, 'Geomorphons'],
+['climate_tools', 23, 'PhenIps (Grids, Days)'],
+['climate_tools', 24, 'Soil Water Balance (Days)'],
+['grid_analysis', 22, 'Shannon Index'],
+['grid_analysis', 23, 'Simpson Index'],
+['grid_analysis', 24, 'Rao\'s Q Diversity Index (Classic)'],
+['grid_analysis', 25, 'Rao\'s Q Diversity Index'],
+['io_gdal', 12, 'Create Virtual Raster (VRT)'],
+['io_gdal', 13, 'Import from Virtual Raster (VRT)'],
+['io_shapes', 23, 'Import Point Cloud from PTS File'],
+['shapes_points', 22, 'Snap Points to Polygons'],
+['sim_hydrology', 8, 'Quasi-Dynamic Flow Accumulation'],
+['statistics_kriging', 5, 'Simple Kriging (3D)'],
+['statistics_kriging', 6, 'Ordinary Kriging (3D)'],
+['ta_cliffmetrics', 0, 'CliffMetrics'],
+['ta_morphometry', 29, 'Wind Shelter Index'],
+['climate_tools', 25, 'Cloud Overlap'],
+['climate_tools', 26, 'Temperature Lapse Rates'],
+['grids_tools', 7, 'Grid Collection Masking'],
+['grid_analysis', 26, 'Coverage of Categories'],
+['imagery_tools', 15, 'Import Sentinel-2 Scene'],
+['io_grid', 18, 'Import Regular SubsurfaceViewer Grid'],
+['pj_georeference', 7, 'Georeference with Coordinate Grids'],
+['pj_proj4', 30, 'Coordinate Conversion (Grids)'],
+['pj_proj4', 31, 'Coordinate Conversion (Table)'],
+['shapes_points', 23, '3D Points Selection'],
+['io_webservices', 0, 'Geocoding'],
+['pj_proj4', 32, 'Globe Gores'],
+['climate_tools', 27, 'Air Pressure Adjustment'],
+['shapes_points', 24, 'Point to Line Distances'],
+['sim_air_flow', 0, 'Cold Air Flow'],
+['sim_cellular_automata', 2, 'Hodgepodge Machine'],
+['sim_hydrology', 9, 'Overland Flow'],
+['sim_hydrology', 10, 'Soil Water Simulation after Glugla (Table)'],
+['sim_hydrology', 11, 'Soil Water Simulation after Glugla (Grid)'],
+['table_calculus', 19, 'Confusion Matrix (Table Fields)'],
+['ta_hydrology', 30, 'Isochrones Variable Speed'],
+['ta_preprocessor', 7, 'Breach Depressions'],
+['climate_tools', 28, 'Land Surface Temperature'],
+['statistics_grid', 20, 'Unique Value Statistics for Grids'],
+['grid_analysis', 27, 'Soil Water Capacity'],
+['grid_analysis', 28, 'Soil Water Capacity (Grid Collections)'],
+['grid_analysis', 29, 'Iterative Truncation'],
+['io_pdal', 0, 'Import Point Cloud'],
+['shapes_tools', 30, 'Gradient Lines from Points'],
+['ta_cliffmetrics', 1, 'Coastal Profile Crossings']
 ]
