@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id$
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -53,6 +50,8 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
+#ifndef HEADER_INCLUDED__SAGA_API__api_core_H
+#define HEADER_INCLUDED__SAGA_API__api_core_H
 
 
 ///////////////////////////////////////////////////////////
@@ -62,8 +61,15 @@
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#ifndef HEADER_INCLUDED__SAGA_API__api_core_H
-#define HEADER_INCLUDED__SAGA_API__api_core_H
+/** \file api_core.h
+* Classes, functions, type definitions for basic helpers 
+* like data types, memory allocation, string handling, file
+* access, translations, colors and user interface communication.
+* @see CSG_String
+* @see CSG_File
+* @see CSG_Array
+* @see CSG_Bytes
+*/
 
 
 ///////////////////////////////////////////////////////////
@@ -172,6 +178,7 @@
 SAGA_API_DLL_EXPORT void			SG_OMP_Set_Max_Num_Threads	(int iCores);
 SAGA_API_DLL_EXPORT int				SG_OMP_Get_Max_Num_Threads	(void);
 SAGA_API_DLL_EXPORT int				SG_OMP_Get_Max_Num_Procs	(void);
+SAGA_API_DLL_EXPORT int				SG_OMP_Get_Thread_Num		(void);
 
 
 ///////////////////////////////////////////////////////////
@@ -195,6 +202,7 @@ SAGA_API_DLL_EXPORT void *			SG_Realloc			(void *memblock, size_t size);
 SAGA_API_DLL_EXPORT void			SG_Free				(void *memblock);
 
 #define SG_FREE_SAFE(PTR)			{ if( PTR ) { SG_Free (PTR); PTR = NULL; } }
+#define SG_DELETE_SAFE(PTR)			{ if( PTR ) { delete  (PTR); PTR = NULL; } }
 #define SG_DELETE_ARRAY(PTR)		{ if( PTR ) { delete[](PTR); PTR = NULL; } }
 
 //---------------------------------------------------------
@@ -208,249 +216,73 @@ SAGA_API_DLL_EXPORT void			SG_Mem_Set_Double	(char *Buffer, double Value, bool b
 
 ///////////////////////////////////////////////////////////
 //														 //
-//						String							 //
-//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define SG_Char				wchar_t
-#define SG_T(s)				L ## s
-#define SG_PRINTF			SG_Printf
-#define SG_FPRINTF			SG_FPrintf
-#define SG_SSCANF			swscanf
-#define SG_STR_CPY			wcscpy
-#define SG_STR_LEN			wcslen
-#define SG_STR_TOD			wcstod
-#define SG_STR_CMP(s1, s2)	CSG_String(s1).Cmp(s2)
-#define SG_STR_MBTOSG(s)	CSG_String(s).w_str()
-
-//---------------------------------------------------------
-class SAGA_API_DLL_EXPORT CSG_String
+class SAGA_API_DLL_EXPORT CSG_Buffer
 {
 public:
-	CSG_String(void);
-	CSG_String(const CSG_String &String);
-	CSG_String(const char       *String);
-	CSG_String(const wchar_t    *String);
-	CSG_String(char    Character, size_t nRepeat = 1);
-	CSG_String(wchar_t Character, size_t nRepeat = 1);
-	CSG_String(const class wxString *pString);
 
-	virtual ~CSG_String(void);
+						CSG_Buffer		(void);
+	bool				Create			(void);
 
-	CSG_String &					operator =			(const CSG_String &String);
-	CSG_String &					operator =			(const char       *String);
-	CSG_String &					operator =			(const wchar_t    *String);
-	CSG_String &					operator =			(char    Character);
-	CSG_String &					operator =			(wchar_t Character);
+						CSG_Buffer		(const CSG_Buffer &Buffer);
+	bool				Create			(const CSG_Buffer &Buffer);
 
-	size_t							Length				(void)	const;
+						CSG_Buffer		(size_t Size);
+	bool				Create			(size_t Size);
 
-	bool							is_Empty			(void)	const;
+	virtual				~CSG_Buffer		(void);
+	void				Destroy			(void);
 
-	SG_Char							operator []			(int    i)	const;
-	SG_Char							operator []			(size_t i)	const;
-	SG_Char							Get_Char			(size_t i)	const;
+	bool				Set_Size		(size_t Size, bool bShrink = true);
+	bool				Inc_Size		(size_t Size)				{	return( Set_Size(m_Size + Size) );	}
+	size_t				Get_Size		(void)				const	{	return( m_Size );	}
 
-	void							Set_Char			(size_t i, char    Character);
-	void							Set_Char			(size_t i, wchar_t Character);
+	bool				Set_Data		(const char *Data, size_t Size, bool bShrink = true);
+	char *				Get_Data		(int Offset = 0)	const	{	return( m_Data + Offset );		}
 
-	const char *					b_str				(void)	const;
-	const wchar_t *					w_str				(void)	const;
-	const SG_Char *					c_str				(void)	const;
+	CSG_Buffer &		operator =		(const CSG_Buffer &Buffer)	{	Create(Buffer);	return( *this );	}
+	char &				operator []		(int Position)				{	return( m_Data[Position] );	}
+	char				operator []		(int Position)		const	{	return( m_Data[Position] );	}
 
-	operator const char *								(void)	const	{	return( b_str() );	}
-	operator const wchar_t *							(void)	const	{	return( w_str() );	}
+	void				Add_Value		(char   Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
+	void				Add_Value		(short  Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
+	void				Add_Value		(int    Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
+	void				Add_Value		(float  Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
+	void				Add_Value		(double Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
 
-	CSG_String &					Prepend				(const CSG_String &String);
+	CSG_Buffer &		operator +=		(char   Value)				{	Add_Value(Value);	return( *this );	}
+	CSG_Buffer &		operator +=		(short  Value)				{	Add_Value(Value);	return( *this );	}
+	CSG_Buffer &		operator +=		(int    Value)				{	Add_Value(Value);	return( *this );	}
+	CSG_Buffer &		operator +=		(float  Value)				{	Add_Value(Value);	return( *this );	}
+	CSG_Buffer &		operator +=		(double Value)				{	Add_Value(Value);	return( *this );	}
 
-	CSG_String &					Append				(const CSG_String &String);
-	CSG_String &					Append				(const char       *String);
-	CSG_String &					Append				(const wchar_t    *String);
-	CSG_String &					Append				(char    Character, size_t nRepeat = 1);
-	CSG_String &					Append				(wchar_t Character, size_t nRepeat = 1);
+	void				Set_Value		(int Offset, char   Value, bool bBigEndian = false)	{	m_Data[Offset]	= Value;	}
+	void				Set_Value		(int Offset, short  Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(short  *)(m_Data + Offset) = Value;	}
+	void				Set_Value		(int Offset, int    Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(int    *)(m_Data + Offset) = Value;	}
+	void				Set_Value		(int Offset, float  Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(float  *)(m_Data + Offset) = Value;	}
+	void				Set_Value		(int Offset, double Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(double *)(m_Data + Offset) = Value;	}
 
-	void							operator +=			(const CSG_String &String);
-	void							operator +=			(const char       *String);
-	void							operator +=			(const wchar_t    *String);
-	void							operator +=			(char    Character);
-	void							operator +=			(wchar_t Character);
-
-	CSG_String						operator +			(const CSG_String &String)	const;
-	CSG_String						operator +			(const char       *String)	const;
-	CSG_String						operator +			(const wchar_t    *String)	const;
-	CSG_String						operator +			(char    Character)			const;
-	CSG_String						operator +			(wchar_t Character)			const;
-
-	int								Cmp					(const CSG_String &String)	const;
-	int								CmpNoCase			(const CSG_String &String)	const;
-
-	bool							is_Same_As			(const CSG_String &String, bool bCase = true)	const;
-	bool							is_Same_As			(const char     Character, bool bCase = true)	const;
-	bool							is_Same_As			(const wchar_t  Character, bool bCase = true)	const;
-
-	CSG_String &					Make_Lower			(void);
-	CSG_String &					Make_Upper			(void);
-
-	void							Clear				(void);
-
-	static CSG_String				Format				(const char    *Format, ...);
-	static CSG_String				Format				(const wchar_t *Format, ...);
-	int								Printf				(const char    *Format, ...);
-	int								Printf				(const wchar_t *Format, ...);
-
-	size_t							Replace				(const CSG_String &sOld, const CSG_String &sNew, bool bReplaceAll = true);
-
-	CSG_String &					Remove				(size_t pos);
-	CSG_String &					Remove				(size_t pos, size_t len);
-
-	int								Trim				(bool fromRight = false);
-	int								Trim_Both			(void);
-
-	int								Find				(char    Character, bool fromEnd = false)	const;
-	int								Find				(wchar_t Character, bool fromEnd = false)	const;
-	int								Find				(const CSG_String &String)			const;
-	bool							Contains			(const CSG_String &String)			const;
-
-	CSG_String						AfterFirst			(char    Character)					const;
-	CSG_String						AfterFirst			(wchar_t Character)					const;
-	CSG_String						AfterLast			(char    Character)					const;
-	CSG_String						AfterLast			(wchar_t Character)					const;
-	CSG_String						BeforeFirst			(char    Character)					const;
-	CSG_String						BeforeFirst			(wchar_t Character)					const;
-	CSG_String						BeforeLast			(char    Character)					const;
-	CSG_String						BeforeLast			(wchar_t Character)					const;
-
-	CSG_String						Right				(size_t count)						const;
-	CSG_String						Mid					(size_t first, size_t count = 0)	const;
-	CSG_String						Left				(size_t count)						const;
-
-	bool							is_Number			(void)	const;
-
-	int								asInt				(void)								const;
-	bool							asInt				(int    &Value)						const;
-
-	double							asDouble			(void)								const;
-	bool							asDouble			(double &Value)						const;
-
-	bool							to_ASCII			(char **pString)					const;
-
-
-protected:
-
-	class wxString					*m_pString;
-
-};
-
-//---------------------------------------------------------
-SAGA_API_DLL_EXPORT CSG_String		operator +			(const char    *A, const CSG_String &B);
-SAGA_API_DLL_EXPORT CSG_String		operator +			(const wchar_t *A, const CSG_String &B);
-SAGA_API_DLL_EXPORT CSG_String		operator +			(char           A, const CSG_String &B);
-SAGA_API_DLL_EXPORT CSG_String		operator +			(wchar_t        A, const CSG_String &B);
-
-//---------------------------------------------------------
-class SAGA_API_DLL_EXPORT CSG_Strings
-{
-public:
-	CSG_Strings(void);
-	CSG_Strings(const CSG_Strings &Strings);
-	CSG_Strings(int nStrings, const SG_Char **Strings);
-
-	virtual ~CSG_Strings(void);
-
-	void							Clear				(void);
-
-	CSG_Strings &					operator  =			(const CSG_Strings &Strings);
-	bool							Assign				(const CSG_Strings &Strings);
-
-	bool							Add					(const CSG_String &String);
-	CSG_Strings &					operator +=			(const CSG_String &String);
-
-	bool							Del					(int Index);
-
-	bool							Set_Count			(int nStrings);
-	int								Get_Count			(void)		const	{	return( m_nStrings );	}
-
-	CSG_String &					operator []			(int Index) const	{	return( *m_Strings[Index]   );	}
-	CSG_String &					Get_String			(int Index) const	{	return( *m_Strings[Index]   );	}
-
-
-protected:
-
-	int								m_nStrings;
-
-	CSG_String						**m_Strings;
-
-};
-
-//---------------------------------------------------------
-#define SG_DEFAULT_DELIMITERS		" \t\r\n"
-
-typedef enum
-{
-	SG_TOKEN_INVALID,
-	SG_TOKEN_DEFAULT,
-	SG_TOKEN_RET_EMPTY,
-	SG_TOKEN_RET_EMPTY_ALL,
-	SG_TOKEN_RET_DELIMS,
-	SG_TOKEN_STRTOK
-}
-TSG_String_Tokenizer_Mode;
-
-//---------------------------------------------------------
-class SAGA_API_DLL_EXPORT CSG_String_Tokenizer
-{
-public:
-	CSG_String_Tokenizer(void);
-	CSG_String_Tokenizer(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
-
-	~CSG_String_Tokenizer(void);
-
-  
-	size_t							Get_Tokens_Count	(void)	const;
-	SG_Char							Get_Last_Delimiter	(void)	const;
-	CSG_String						Get_Next_Token		(void);
-	size_t							Get_Position		(void)	const;
-	CSG_String						Get_String			(void)	const;
-	bool							Has_More_Tokens		(void)	const;
-	void							Set_String			(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
+	short				asShort			(int Offset, bool bBigEndian = false) const	{	short  Value = *(short  *)(m_Data + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
+	int					asInt			(int Offset, bool bBigEndian = false) const	{	int    Value = *(int    *)(m_Data + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
+	float				asFloat			(int Offset, bool bBigEndian = false) const	{	float  Value = *(float  *)(m_Data + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
+	double				asDouble		(int Offset, bool bBigEndian = false) const	{	double Value = *(double *)(m_Data + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
 
 
 private:
 
-	class wxStringTokenizer			*m_pTokenizer;
+	char				*m_Data;
+
+	size_t				m_Size;
 
 };
 
-//---------------------------------------------------------
-SAGA_API_DLL_EXPORT CSG_Strings		SG_String_Tokenize				(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
-
-//---------------------------------------------------------
-SAGA_API_DLL_EXPORT bool			SG_is_Character_Numeric			(int Character);
-
-SAGA_API_DLL_EXPORT int				SG_Printf						(const CSG_String &String);
-SAGA_API_DLL_EXPORT int				SG_Printf						(const  char   *Format, ...);
-SAGA_API_DLL_EXPORT int				SG_Printf						(const wchar_t *Format, ...);
-
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const CSG_String &String);
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const  char   *Format, ...);
-SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const wchar_t *Format, ...);
-
-SAGA_API_DLL_EXPORT CSG_String		SG_Get_CurrentTimeStr			(bool bWithDate = true);
-
-SAGA_API_DLL_EXPORT double			SG_Degree_To_Double				(const CSG_String &String);
-SAGA_API_DLL_EXPORT CSG_String		SG_Double_To_Degree				(double Value);
-
-SAGA_API_DLL_EXPORT int				SG_Get_Significant_Decimals		(double Value, int maxDecimals = 6);
-
-SAGA_API_DLL_EXPORT void			SG_Flip_Decimal_Separators		(CSG_String &String);
-
-SAGA_API_DLL_EXPORT CSG_String		SG_Get_String					(double Value, int Precision = -99);
-SAGA_API_DLL_EXPORT CSG_String		SG_Get_String					(int    Value, int Precision = 0);
-
 
 ///////////////////////////////////////////////////////////
 //														 //
+//														 //
+//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -470,10 +302,10 @@ public:
 	CSG_Array(void);
 	~CSG_Array(void);
 
-						CSG_Array		(const CSG_Array &Array);
+	CSG_Array		(const CSG_Array &Array);
 	void *				Create			(const CSG_Array &Array);
 
-						CSG_Array		(size_t Value_Size, size_t nValues = 0, TSG_Array_Growth Growth = SG_ARRAY_GROWTH_0);
+	CSG_Array		(size_t Value_Size, size_t nValues = 0, TSG_Array_Growth Growth = SG_ARRAY_GROWTH_0);
 	void *				Create			(size_t Value_Size, size_t nValues = 0, TSG_Array_Growth Growth = SG_ARRAY_GROWTH_0);
 
 	void				Destroy			(void);
@@ -641,68 +473,285 @@ private:
 
 ///////////////////////////////////////////////////////////
 //														 //
+//						String							 //
+//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-class SAGA_API_DLL_EXPORT CSG_Buffer
+#define SG_Char				wchar_t
+#define SG_T(s)				L ## s
+#define SG_PRINTF			SG_Printf
+#define SG_FPRINTF			SG_FPrintf
+#define SG_SSCANF			swscanf
+#define SG_STR_CPY			wcscpy
+#define SG_STR_LEN			wcslen
+#define SG_STR_TOD			wcstod
+#define SG_STR_CMP(s1, s2)	CSG_String(s1).Cmp(s2)
+#define SG_STR_MBTOSG(s)	CSG_String(s).w_str()
+
+//---------------------------------------------------------
+enum ESG_File_Flags_Encoding
+{
+	SG_FILE_ENCODING_ANSI,
+	SG_FILE_ENCODING_UTF7,
+	SG_FILE_ENCODING_UTF8,
+	SG_FILE_ENCODING_UTF16LE,
+	SG_FILE_ENCODING_UTF16BE,
+	SG_FILE_ENCODING_UTF32LE,
+	SG_FILE_ENCODING_UTF32BE,
+	SG_FILE_ENCODING_UNDEFINED
+};
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_String
 {
 public:
+	CSG_String(void);
+	CSG_String(const CSG_String &String);
+	CSG_String(const char       *String);
+	CSG_String(const wchar_t    *String);
+	CSG_String(char    Character, size_t nRepeat = 1);
+	CSG_String(wchar_t Character, size_t nRepeat = 1);
 
-						CSG_Buffer		(void);
-	bool				Create			(void);
+									CSG_String			(const class wxString *pString);
+	bool							Create				(const class wxString *pString);
 
-						CSG_Buffer		(const CSG_Buffer &Buffer);
-	bool				Create			(const CSG_Buffer &Buffer);
+	virtual ~CSG_String(void);
 
-						CSG_Buffer		(size_t Size);
-	bool				Create			(size_t Size);
+	CSG_String &					operator =			(const CSG_String &String);
+	CSG_String &					operator =			(const char       *String);
+	CSG_String &					operator =			(const wchar_t    *String);
+	CSG_String &					operator =			(char    Character);
+	CSG_String &					operator =			(wchar_t Character);
 
-	virtual				~CSG_Buffer		(void);
-	void				Destroy			(void);
+	size_t							Length				(void)	const;
 
-	bool				Set_Size		(size_t Size, bool bShrink = true);
-	bool				Inc_Size		(size_t Size)				{	return( Set_Size(m_Size + Size) );	}
-	size_t				Get_Size		(void)				const	{	return( m_Size );	}
+	bool							is_Empty			(void)	const;
 
-	bool				Set_Data		(const char *Buffer, size_t Size, bool bShrink = true);
-	char *				Get_Data		(int Offset = 0)	const	{	return( m_pData + Offset );		}
-	char				operator []		(int Position)		const	{	return( m_pData[Position] );	}
+	SG_Char							operator []			(int    i)	const;
+	SG_Char							operator []			(size_t i)	const;
+	SG_Char							Get_Char			(size_t i)	const;
 
-	void				Add_Value		(char   Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
-	void				Add_Value		(short  Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
-	void				Add_Value		(int    Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
-	void				Add_Value		(float  Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
-	void				Add_Value		(double Value, bool bBigEndian = false)	{	if( Inc_Size(sizeof(Value)) ) Set_Value((int)m_Size - sizeof(Value), Value, bBigEndian);	}
+	void							Set_Char			(size_t i, char    Character);
+	void							Set_Char			(size_t i, wchar_t Character);
 
-	CSG_Buffer &		operator +=		(char   Value)				{	Add_Value(Value);	return( *this );	}
-	CSG_Buffer &		operator +=		(short  Value)				{	Add_Value(Value);	return( *this );	}
-	CSG_Buffer &		operator +=		(int    Value)				{	Add_Value(Value);	return( *this );	}
-	CSG_Buffer &		operator +=		(float  Value)				{	Add_Value(Value);	return( *this );	}
-	CSG_Buffer &		operator +=		(double Value)				{	Add_Value(Value);	return( *this );	}
+	const char *					b_str				(void)	const;
+	const wchar_t *					w_str				(void)	const;
+	const SG_Char *					c_str				(void)	const;
 
-	void				Set_Value		(int Offset, char   Value, bool bBigEndian = false)	{	m_pData[Offset]	= Value;	}
-	void				Set_Value		(int Offset, short  Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(short  *)(m_pData + Offset) = Value;	}
-	void				Set_Value		(int Offset, int    Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(int    *)(m_pData + Offset) = Value;	}
-	void				Set_Value		(int Offset, float  Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(float  *)(m_pData + Offset) = Value;	}
-	void				Set_Value		(int Offset, double Value, bool bBigEndian = false)	{	if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); *(double *)(m_pData + Offset) = Value;	}
+	operator const char *								(void)	const	{	return( b_str() );	}
+	operator const wchar_t *							(void)	const	{	return( w_str() );	}
 
-	short				asShort			(int Offset, bool bBigEndian = false) const	{	short  Value = *(short  *)(m_pData + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
-	int					asInt			(int Offset, bool bBigEndian = false) const	{	int    Value = *(int    *)(m_pData + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
-	float				asFloat			(int Offset, bool bBigEndian = false) const	{	float  Value = *(float  *)(m_pData + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
-	double				asDouble		(int Offset, bool bBigEndian = false) const	{	double Value = *(double *)(m_pData + Offset); if( bBigEndian ) SG_Swap_Bytes(&Value, sizeof(Value)); return( Value );	}
+	CSG_String &					Prepend				(const CSG_String &String);
+
+	CSG_String &					Append				(const CSG_String &String);
+	CSG_String &					Append				(const char       *String);
+	CSG_String &					Append				(const wchar_t    *String);
+	CSG_String &					Append				(char    Character, size_t nRepeat = 1);
+	CSG_String &					Append				(wchar_t Character, size_t nRepeat = 1);
+
+	void							operator +=			(const CSG_String &String);
+	void							operator +=			(const char       *String);
+	void							operator +=			(const wchar_t    *String);
+	void							operator +=			(char    Character);
+	void							operator +=			(wchar_t Character);
+
+	CSG_String						operator +			(const CSG_String &String)	const;
+	CSG_String						operator +			(const char       *String)	const;
+	CSG_String						operator +			(const wchar_t    *String)	const;
+	CSG_String						operator +			(char    Character)			const;
+	CSG_String						operator +			(wchar_t Character)			const;
+
+	int								Cmp					(const CSG_String &String)	const;
+	int								CmpNoCase			(const CSG_String &String)	const;
+
+	bool							is_Same_As			(const CSG_String &String, bool bCase = true)	const;
+	bool							is_Same_As			(const char     Character, bool bCase = true)	const;
+	bool							is_Same_As			(const wchar_t  Character, bool bCase = true)	const;
+
+	CSG_String &					Make_Lower			(void);
+	CSG_String &					Make_Upper			(void);
+
+	void							Clear				(void);
+
+	static CSG_String				Format				(const char    *Format, ...);
+	static CSG_String				Format				(const wchar_t *Format, ...);
+	int								Printf				(const char    *Format, ...);
+	int								Printf				(const wchar_t *Format, ...);
+
+	size_t							Replace				(const CSG_String &sOld, const CSG_String &sNew, bool bReplaceAll = true);
+
+	CSG_String &					Remove				(size_t pos);
+	CSG_String &					Remove				(size_t pos, size_t len);
+
+	int								Trim				(bool fromRight = false);
+	int								Trim_Both			(void);
+
+	int								Find				(char    Character, bool fromEnd = false)	const;
+	int								Find				(wchar_t Character, bool fromEnd = false)	const;
+	int								Find				(const CSG_String &String)				const;
+	bool							Contains			(const CSG_String &String)				const;
+
+	CSG_String						AfterFirst			(char    Character)						const;
+	CSG_String						AfterFirst			(wchar_t Character)						const;
+	CSG_String						AfterLast			(char    Character)						const;
+	CSG_String						AfterLast			(wchar_t Character)						const;
+	CSG_String						BeforeFirst			(char    Character)						const;
+	CSG_String						BeforeFirst			(wchar_t Character)						const;
+	CSG_String						BeforeLast			(char    Character)						const;
+	CSG_String						BeforeLast			(wchar_t Character)						const;
+
+	CSG_String						Right				(size_t count)							const;
+	CSG_String						Mid					(size_t first, size_t count = 0)		const;
+	CSG_String						Left				(size_t count)							const;
+
+	bool							is_Number			(void)	const;
+
+	int								asInt				(void)									const;
+	bool							asInt				(int    &Value)							const;
+
+	double							asDouble			(void)									const;
+	bool							asDouble			(double &Value)							const;
+
+	static CSG_String				from_UTF8			(const char *String, size_t Length = 0);
+
+	size_t							to_UTF8				(char **pString)						const;
+	CSG_Buffer						to_UTF8				(void)									const;
+	size_t							to_MBChar			(char **pString, int Encoding)			const;
+	CSG_Buffer						to_MBChar			(                int Encoding)			const;
+	bool							to_ASCII			(char **pString, char Replace = '_')	const;
+	CSG_Buffer						to_ASCII			(                char Replace = '_')	const;
+
+
+protected:
+
+	class wxString					*m_pString;
+
+};
+
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT CSG_String		operator +			(const char    *A, const CSG_String &B);
+SAGA_API_DLL_EXPORT CSG_String		operator +			(const wchar_t *A, const CSG_String &B);
+SAGA_API_DLL_EXPORT CSG_String		operator +			(char           A, const CSG_String &B);
+SAGA_API_DLL_EXPORT CSG_String		operator +			(wchar_t        A, const CSG_String &B);
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_Strings
+{
+public:
+	CSG_Strings(void);
+	CSG_Strings(const CSG_Strings &Strings);
+	CSG_Strings(int nStrings, const SG_Char **Strings);
+
+	virtual ~CSG_Strings(void);
+
+	bool							Create				(const CSG_Strings &Strings);
+	bool							Destroy				(void);
+
+	bool							Set_Count			(int    Count)	{	return( Set_Count((size_t)Count) );	}
+	bool							Set_Count			(size_t Count);
+	int								Get_Count			(void)	const	{	return( (int)m_Strings.Get_Size() );	}
+	size_t							Get_Size			(void)	const	{	return(      m_Strings.Get_Size() );	}
+
+	bool							Add					(const CSG_Strings &Strings);
+	bool							Add					(const CSG_String  &String );
+	bool							Ins					(const CSG_String  &String , int    Index)	{	return( Ins(String, (size_t)Index) );	}
+	bool							Ins					(const CSG_String  &String , size_t Index);
+	bool							Del					(int    Index)	{	return( Del((size_t)Index) );	}
+	bool							Del					(size_t Index);
+
+	CSG_String &					Get_String			(int    Index) const	{	return( *((CSG_String *)m_Strings[Index]) );	}
+	CSG_String &					Get_String			(size_t Index) const	{	return( *((CSG_String *)m_Strings[Index]) );	}
+
+	CSG_String &					operator []			(int    Index) const	{	return( Get_String(Index) );	}
+	CSG_String &					operator []			(size_t Index) const	{	return( Get_String(Index) );	}
+
+	CSG_Strings &					operator  =			(const CSG_Strings &Strings)	{	Create(Strings); return( *this );	}
+	CSG_Strings &					operator +=			(const CSG_Strings &Strings)	{	Add   (Strings); return( *this );	}
+	CSG_Strings &					operator +=			(const CSG_String  &String )	{	Add   (String ); return( *this );	}
+
+	bool							Sort				(bool Ascending = true);
+
+	void							Clear				(void)	{	Destroy();	}
+	bool							Assign				(const CSG_Strings &Strings)	{	return( Create(Strings) );	}
+
+
+protected:
+
+	CSG_Array_Pointer				m_Strings;
+
+};
+
+//---------------------------------------------------------
+#define SG_DEFAULT_DELIMITERS		" \t\r\n"
+
+typedef enum
+{
+	SG_TOKEN_INVALID,
+	SG_TOKEN_DEFAULT,
+	SG_TOKEN_RET_EMPTY,
+	SG_TOKEN_RET_EMPTY_ALL,
+	SG_TOKEN_RET_DELIMS,
+	SG_TOKEN_STRTOK
+}
+TSG_String_Tokenizer_Mode;
+
+//---------------------------------------------------------
+class SAGA_API_DLL_EXPORT CSG_String_Tokenizer
+{
+public:
+	CSG_String_Tokenizer(void);
+	CSG_String_Tokenizer(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
+
+	~CSG_String_Tokenizer(void);
+
+  
+	size_t							Get_Tokens_Count	(void)	const;
+	SG_Char							Get_Last_Delimiter	(void)	const;
+	CSG_String						Get_Next_Token		(void);
+	size_t							Get_Position		(void)	const;
+	CSG_String						Get_String			(void)	const;
+	bool							Has_More_Tokens		(void)	const;
+	void							Set_String			(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
 
 
 private:
 
-	char				*m_pData;
-
-	size_t				m_Size;
+	class wxStringTokenizer			*m_pTokenizer;
 
 };
 
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT CSG_Strings		SG_String_Tokenize				(const CSG_String &String, const CSG_String &Delimiters = SG_DEFAULT_DELIMITERS, TSG_String_Tokenizer_Mode Mode = SG_TOKEN_DEFAULT);
+
+//---------------------------------------------------------
+SAGA_API_DLL_EXPORT bool			SG_is_Character_Numeric			(int Character);
+
+SAGA_API_DLL_EXPORT int				SG_Printf						(const CSG_String &String);
+SAGA_API_DLL_EXPORT int				SG_Printf						(const  char   *Format, ...);
+SAGA_API_DLL_EXPORT int				SG_Printf						(const wchar_t *Format, ...);
+
+SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const CSG_String &String);
+SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const  char   *Format, ...);
+SAGA_API_DLL_EXPORT int				SG_FPrintf						(FILE *Stream, const wchar_t *Format, ...);
+
+SAGA_API_DLL_EXPORT CSG_String		SG_Get_CurrentTimeStr			(bool bWithDate = true);
+
+SAGA_API_DLL_EXPORT double			SG_Degree_To_Double				(const CSG_String &String);
+SAGA_API_DLL_EXPORT CSG_String		SG_Double_To_Degree				(double Value);
+
+SAGA_API_DLL_EXPORT int				SG_Get_Significant_Decimals		(double Value, int maxDecimals = 6);
+
+SAGA_API_DLL_EXPORT void			SG_Flip_Decimal_Separators		(CSG_String &String);
+
+SAGA_API_DLL_EXPORT CSG_String		SG_Get_String					(double Value, int Precision = -99);
+SAGA_API_DLL_EXPORT CSG_String		SG_Get_String					(int    Value, int Precision = 0);
+
 
 ///////////////////////////////////////////////////////////
 //														 //
+//														 //
+//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -820,6 +869,8 @@ private:
 
 ///////////////////////////////////////////////////////////
 //														 //
+//														 //
+//														 //
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
@@ -1001,15 +1052,6 @@ enum ESG_File_Flags_Open
 	SG_FILE_RW
 };
 
-//---------------------------------------------------------
-enum ESG_File_Flags_Encoding
-{
-	SG_FILE_ENCODING_CHAR,
-	SG_FILE_ENCODING_UNICODE,
-	SG_FILE_ENCODING_UTF8,
-	SG_FILE_ENCODING_UTF16
-};
-
 //---------------------------------------------------------
 enum ESG_File_Flags_Seek
 {
@@ -1026,8 +1068,8 @@ public:
 	CSG_File(void);
 	virtual ~CSG_File(void);
 
-									CSG_File			(const CSG_String &FileName, int Mode = SG_FILE_R, bool bBinary = true, int Encoding = SG_FILE_ENCODING_CHAR);
-	virtual bool					Open				(const CSG_String &FileName, int Mode = SG_FILE_R, bool bBinary = true, int Encoding = SG_FILE_ENCODING_CHAR);
+									CSG_File			(const CSG_String &FileName, int Mode = SG_FILE_R, bool bBinary = true, int Encoding = SG_FILE_ENCODING_ANSI);
+	virtual bool					Open				(const CSG_String &FileName, int Mode = SG_FILE_R, bool bBinary = true, int Encoding = SG_FILE_ENCODING_ANSI);
 
 	virtual bool					Close				(void);
 
@@ -1036,6 +1078,7 @@ public:
 
 	class wxStreamBase *			Get_Stream			(void)	const	{	return( (class wxStreamBase *)m_pStream );	}
 
+	bool							Set_Encoding		(int Encoding);
 	int								Get_Encoding		(void)	const	{	return( m_Encoding );	}
 
 	bool							is_Open				(void)	const	{	return( m_pStream != NULL );	}
@@ -1082,7 +1125,10 @@ protected:
 
 	CSG_String						m_FileName;
 
-	void							*m_pStream;
+	void							*m_pStream, *m_pConvert;
+
+
+	void							On_Construction		(void);
 
 };
 
@@ -1094,8 +1140,8 @@ public:
 	CSG_File_Zip(void);
 	virtual ~CSG_File_Zip(void);
 
-									CSG_File_Zip		(const CSG_String &FileName, int Mode = SG_FILE_R);
-	virtual bool					Open				(const CSG_String &FileName, int Mode = SG_FILE_R);
+									CSG_File_Zip		(const CSG_String &FileName, int Mode = SG_FILE_R, int Encoding = SG_FILE_ENCODING_ANSI);
+	virtual bool					Open				(const CSG_String &FileName, int Mode = SG_FILE_R, int Encoding = SG_FILE_ENCODING_ANSI);
 
 	virtual bool					Close				(void);
 
@@ -1120,6 +1166,7 @@ protected:
 //---------------------------------------------------------
 SAGA_API_DLL_EXPORT bool			SG_Dir_Exists				(const CSG_String &Directory);
 SAGA_API_DLL_EXPORT bool			SG_Dir_Create				(const CSG_String &Directory);
+SAGA_API_DLL_EXPORT bool			SG_Dir_Delete				(const CSG_String &Directory, bool bRecursive = false);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Current			(void);
 SAGA_API_DLL_EXPORT CSG_String		SG_Dir_Get_Temp				(void);
 SAGA_API_DLL_EXPORT bool			SG_Dir_List_Subdirectories	(CSG_Strings &List, const CSG_String &Directory);
@@ -1152,7 +1199,7 @@ SAGA_API_DLL_EXPORT bool			SG_Set_Environment			(const CSG_String &Variable, con
 ///////////////////////////////////////////////////////////
 
 //---------------------------------------------------------
-#define SG_GET_RGB(r, g, b)		((DWORD) (((BYTE)(r) | ((WORD)(g) << 8)) | (((DWORD)(BYTE)(b)) << 16)))
+#define SG_GET_RGB( r, g, b   )	((DWORD) (((BYTE)(r) | ((WORD)(g) << 8)) | (((DWORD)(BYTE)(b)) << 16)))
 #define SG_GET_RGBA(r, g, b, a)	((DWORD) (((BYTE)(r) | ((WORD)(g) << 8)) | (((DWORD)(BYTE)(b)) << 16) | (((DWORD)(BYTE)(a)) << 24)))
 
 #define SG_GET_R(rgb)			((BYTE) ((rgb)      ))
@@ -1208,6 +1255,9 @@ enum ESG_Colors
 	SG_COLORS_RAINBOW,
 	SG_COLORS_NEON,
 	SG_COLORS_TOPOGRAPHY,
+	SG_COLORS_TOPOGRAPHY_2,
+	SG_COLORS_TOPOGRAPHY_3,
+	SG_COLORS_PRECIPITATION,
 	SG_COLORS_ASPECT_1,
 	SG_COLORS_ASPECT_2,
 	SG_COLORS_ASPECT_3,
@@ -1215,17 +1265,26 @@ enum ESG_Colors
 };
 
 //---------------------------------------------------------
+SAGA_API_DLL_EXPORT CSG_String		SG_Colors_Get_Name	(int Index);
+
 SAGA_API_DLL_EXPORT long			SG_Color_Get_Random	(void);
 
-SAGA_API_DLL_EXPORT CSG_String		SG_Colors_Get_Name	(int Identifier);
+SAGA_API_DLL_EXPORT bool			SG_Color_From_Text	(const CSG_String &Text, long &Color);
+SAGA_API_DLL_EXPORT CSG_String		SG_Color_To_Text	(long Color, bool bHexadecimal = true);
 
 //---------------------------------------------------------
 class SAGA_API_DLL_EXPORT CSG_Colors
 {
 public:
-	CSG_Colors(void);
-	CSG_Colors(const CSG_Colors &Colors);
-	CSG_Colors(int nColors, int Palette = SG_COLORS_DEFAULT, bool bRevert = false);
+									CSG_Colors			(void);
+	bool							Create				(void);
+
+									CSG_Colors			(const CSG_Colors &Colors);
+	bool							Create				(const CSG_Colors &Colors);
+
+									CSG_Colors			(int nColors, int Palette = SG_COLORS_DEFAULT, bool bRevert = false);
+	bool							Create				(int nColors, int Palette = SG_COLORS_DEFAULT, bool bRevert = false);
+
 	virtual ~CSG_Colors(void);
 
 	void							Destroy				(void);
@@ -1264,8 +1323,11 @@ public:
 		);
 	}
 
+	static int						Get_Predefined_Count(void);
+	static const SG_Char *			Get_Predefined_Name	(int Identifier);
+	bool							Set_Predefined		(int Index, bool bRevert = false, int nColors = 11);
+	bool							Set_Palette			(int Index, bool bRevert = false, int nColors = 11)	{	return( Set_Predefined(Index, bRevert, nColors) );	}
 	bool							Set_Default			(int nColors = 11);
-	bool							Set_Palette			(int Index, bool bRevert = false, int nColors = 11);
 	bool							Set_Ramp			(long Color_A, long Color_B);
 	bool							Set_Ramp			(long Color_A, long Color_B, int iColor_A, int iColor_B);
 	bool							Set_Ramp_Brighness	(int Brightness_A, int Brightness_B);
@@ -1274,6 +1336,7 @@ public:
 	bool							Random				(void);
 	bool							Invert				(void);
 	bool							Revert				(void);
+	bool							Greyscale			(void);
 
 	bool							Assign				(const CSG_Colors &Colors);
 	bool							Assign				(CSG_Colors *pSource);
@@ -1444,12 +1507,12 @@ TSG_UI_Callback_ID;
 class SAGA_API_DLL_EXPORT CSG_UI_Parameter
 {
 public:
-	CSG_UI_Parameter(void)						: Boolean(false), Number( 0.0 ), Pointer(NULL)		{}
-	CSG_UI_Parameter(bool              Value)	: Boolean(Value), Number( 0.0 ), Pointer(NULL)		{}
+	CSG_UI_Parameter(void)						: Boolean(false), Number( 0.  ), Pointer(NULL)		{}
+	CSG_UI_Parameter(bool              Value)	: Boolean(Value), Number( 0.  ), Pointer(NULL)		{}
 	CSG_UI_Parameter(int               Value)	: Boolean(false), Number(Value), Pointer(NULL)		{}
 	CSG_UI_Parameter(double            Value)	: Boolean(false), Number(Value), Pointer(NULL)		{}
-	CSG_UI_Parameter(const CSG_String &Value)	: Boolean(false), Number( 0.0 ), Pointer(NULL), String(Value)	{}
-	CSG_UI_Parameter(void             *Value)	: Boolean(false), Number( 0.0 ), Pointer(Value)	{}
+	CSG_UI_Parameter(const CSG_String &Value)	: Boolean(false), Number( 0.  ), Pointer(NULL), String(Value)	{}
+	CSG_UI_Parameter(void             *Value)	: Boolean(false), Number( 0.  ), Pointer(Value)	{}
 	CSG_UI_Parameter(const CSG_UI_Parameter &Copy);
 
 	bool		Boolean;
