@@ -1,6 +1,3 @@
-/**********************************************************
- * Version $Id$
- *********************************************************/
 
 ///////////////////////////////////////////////////////////
 //                                                       //
@@ -52,15 +49,6 @@
 //                                                       //
 ///////////////////////////////////////////////////////////
 
-//---------------------------------------------------------
-
-
-///////////////////////////////////////////////////////////
-//														 //
-//														 //
-//														 //
-///////////////////////////////////////////////////////////
-
 //---------------------------------------------------------
 #include <math.h>
 
@@ -69,8 +57,10 @@
 #include <wx/tokenzr.h>
 #include <wx/wxcrt.h>
 #include <wx/wxcrtvararg.h>
+#include <wx/convauto.h>
 
 #include "api_core.h"
+#include "mat_tools.h"
 
 
 ///////////////////////////////////////////////////////////
@@ -110,6 +100,7 @@ CSG_String::CSG_String(wchar_t Character, size_t nRepeat)
 	m_pString	= new wxString(Character, nRepeat);
 }
 
+//---------------------------------------------------------
 CSG_String::CSG_String(const class wxString *pString)
 {
 	if( pString )
@@ -118,6 +109,16 @@ CSG_String::CSG_String(const class wxString *pString)
 		m_pString	= new wxString;
 }
 
+bool CSG_String::Create(const class wxString *pString)
+{
+	if( pString )
+		*m_pString	= *pString;
+	else
+		m_pString->Clear();
+
+	return( true );
+}
+
 //---------------------------------------------------------
 CSG_String::~CSG_String(void)
 {
@@ -263,6 +264,11 @@ void CSG_String::Clear(void)
 	m_pString->Clear();
 }
 
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
 //---------------------------------------------------------
 CSG_String CSG_String::Format(const char *Format, ...)
 {
@@ -756,6 +762,129 @@ bool CSG_String::asDouble(double &Value) const
 }
 
 
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+CSG_String CSG_String::from_UTF8(const char *String, size_t Length)
+{
+	CSG_String	s;
+
+	if( String )
+	{
+		if( !Length )
+		{
+			Length	= strlen(String);
+		}
+
+		*s.m_pString	= wxString::FromUTF8(String, Length);
+	}
+
+	return( s );
+}
+
+//---------------------------------------------------------
+/**
+* This function creates a single char array and fills it
+* with the UTF-8 representation of this string object.
+* If successful pString will point to the address of the
+* allocated memory. Freeing memory is in response of the
+* calling function.
+*/
+//---------------------------------------------------------
+size_t CSG_String::to_UTF8(char **pString) const
+{
+	if( !is_Empty() )
+	{
+		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+
+		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
+		{
+			memcpy(*pString, Buffer.data(), Buffer.length());
+
+			return( Buffer.length() );
+		}
+	}
+
+	*pString	= NULL;
+
+	return( 0 );
+}
+
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_UTF8(void) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
+	{
+		const wxScopedCharBuffer	Buffer	= m_pString->utf8_str();
+
+		String.Set_Data(Buffer.data(), Buffer.length() + 1);
+	}
+
+	return( String );
+}
+
+
+///////////////////////////////////////////////////////////
+//														 //
+///////////////////////////////////////////////////////////
+
+//---------------------------------------------------------
+/**
+* This function creates a single char array and fills it
+* with the multibyte representation of this string object.
+* If successful pString will point to the address of the
+* allocated memory. Freeing memory is in response of the
+* calling function.
+*/
+//---------------------------------------------------------
+size_t CSG_String::to_MBChar(char **pString, int Encoding) const
+{
+	CSG_Buffer	String(to_MBChar(Encoding));
+
+	if( String.Get_Size() && (*pString = (char *)SG_Malloc(String.Get_Size())) != NULL )
+	{
+		memcpy(*pString, String.Get_Data(), String.Get_Size());
+
+		return( String.Get_Size() );
+	}
+
+	*pString	= NULL;
+
+	return( 0 );
+}
+
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_MBChar(int Encoding) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
+	{
+		wxScopedCharBuffer	Buffer;
+
+		switch( Encoding )	// selecting the appropriate wxMBConv class
+		{
+		case SG_FILE_ENCODING_ANSI   : Buffer = m_pString->mb_str(wxConvLibc       ); break;
+		case SG_FILE_ENCODING_UTF7   : Buffer = m_pString->mb_str(wxConvUTF7       ); break;
+		case SG_FILE_ENCODING_UTF8   : Buffer = m_pString->mb_str(wxConvUTF8       ); break;
+		case SG_FILE_ENCODING_UTF16LE: Buffer = m_pString->mb_str(wxMBConvUTF16LE()); break;
+		case SG_FILE_ENCODING_UTF16BE: Buffer = m_pString->mb_str(wxMBConvUTF16BE()); break;
+		case SG_FILE_ENCODING_UTF32LE: Buffer = m_pString->mb_str(wxMBConvUTF32LE()); break;
+		case SG_FILE_ENCODING_UTF32BE: Buffer = m_pString->mb_str(wxMBConvUTF32BE()); break;
+		default                      : Buffer = m_pString->mb_str(wxConvAuto     ()); break;
+		}
+
+		String.Set_Data(Buffer.data(), Buffer.length());
+	}
+
+	return( String );
+}
+
+
 ///////////////////////////////////////////////////////////
 //														 //
 ///////////////////////////////////////////////////////////
@@ -769,25 +898,44 @@ bool CSG_String::asDouble(double &Value) const
   * calling function.
 */
 //---------------------------------------------------------
-bool CSG_String::to_ASCII(char **pString)	const
+bool CSG_String::to_ASCII(char **pString, char Replace)	const
 {
-	if( is_Empty() )
+	if( !is_Empty() )
 	{
-		return( false );
+		#if wxCHECK_VERSION(3, 1, 0)
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
+		#else
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
+		#endif
+
+		if( (*pString = (char *)SG_Malloc(Buffer.length())) != NULL )
+		{
+			memcpy(*pString, Buffer.data(), Buffer.length());
+
+			return( true );
+		}
 	}
 
-	*pString	= (char *)SG_Malloc((1 + Length()) * sizeof(char));
+	return( false );
+}
 
-	if( *pString == NULL )
+//---------------------------------------------------------
+CSG_Buffer CSG_String::to_ASCII(char Replace) const
+{
+	CSG_Buffer	String;
+
+	if( !is_Empty() )
 	{
-		return( false );
+		#if wxCHECK_VERSION(3, 1, 0)
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii(Replace);
+		#else
+			const wxScopedCharBuffer	Buffer	= m_pString->ToAscii();
+		#endif
+
+		String.Set_Data(Buffer.data(), Buffer.length());
 	}
 
-	memcpy(*pString, m_pString->ToAscii(), Length() * sizeof(char));
-
-	(*pString)[Length()]	= '\0';
-
-	return( true );
+	return( String );
 }
 
 
@@ -799,26 +947,24 @@ bool CSG_String::to_ASCII(char **pString)	const
 
 //---------------------------------------------------------
 CSG_Strings::CSG_Strings(void)
-{
-	m_nStrings	= 0;
-	m_Strings	= NULL;
-}
+{}
 
 //---------------------------------------------------------
 CSG_Strings::CSG_Strings(const CSG_Strings &Strings)
 {
-	m_nStrings	= 0;
-	m_Strings	= NULL;
+	Create(Strings);
+}
 
-	Assign(Strings);
+bool CSG_Strings::Create(const CSG_Strings &Strings)
+{
+	Destroy();
+
+	return( Add(Strings) );
 }
 
 //---------------------------------------------------------
 CSG_Strings::CSG_Strings(int nStrings, const SG_Char **Strings)
 {
-	m_nStrings	= 0;
-	m_Strings	= NULL;
-
 	for(int i=0; i<nStrings; i++)
 	{
 		Add(Strings[i]);
@@ -832,28 +978,22 @@ CSG_Strings::~CSG_Strings(void)
 }
 
 //---------------------------------------------------------
-void CSG_Strings::Clear(void)
+bool CSG_Strings::Destroy(void)
 {
-	if( m_Strings )
+	for(size_t i=0; i<Get_Size(); i++)
 	{
-		for(int i=0; i<m_nStrings; i++)
-		{
-			delete(m_Strings[i]);
-		}
-
-		SG_Free(m_Strings);
-
-		m_nStrings	= 0;
-		m_Strings	= NULL;
+		delete((CSG_String *)m_Strings[i]);
 	}
+
+	m_Strings.Destroy();
+
+	return( true );
 }
 
 //---------------------------------------------------------
-bool CSG_Strings::Assign(const CSG_Strings &Strings)
+bool CSG_Strings::Add(const CSG_Strings &Strings)
 {
-	Clear();
-
-	for(int i=0; i<Strings.m_nStrings; i++)
+	for(size_t i=0; i<Strings.Get_Size(); i++)
 	{
 		Add(Strings[i]);
 	}
@@ -861,38 +1001,14 @@ bool CSG_Strings::Assign(const CSG_Strings &Strings)
 	return( true );
 }
 
-//---------------------------------------------------------
-CSG_Strings & CSG_Strings::operator  = (const CSG_Strings &Strings)
-{
-	Assign(Strings);
-
-	return( *this );
-}
-
 //---------------------------------------------------------
 bool CSG_Strings::Add(const CSG_String &String)
 {
-	m_Strings	= (CSG_String **)SG_Realloc(m_Strings, (m_nStrings + 1) * sizeof(CSG_String *));
-	m_Strings[m_nStrings++]	= new CSG_String(String);
+	size_t	i	= Get_Size();
 
-	return( true );
-}
-
-//---------------------------------------------------------
-bool CSG_Strings::Del(int Index)
-{
-	if( Index >= 0 && Index < m_nStrings )
+	if( m_Strings.Inc_Array() )
 	{
-		delete(m_Strings[Index]);
-
-		m_nStrings--;
-
-		for(int i=Index; i<m_nStrings; i++)
-		{
-			m_Strings[i]	= m_Strings[i + 1];
-		}
-
-		m_Strings	= (CSG_String **)SG_Realloc(m_Strings, m_nStrings * sizeof(CSG_String *));
+		m_Strings[i]	=  new CSG_String(String);
 
 		return( true );
 	}
@@ -901,21 +1017,95 @@ bool CSG_Strings::Del(int Index)
 }
 
 //---------------------------------------------------------
-CSG_Strings & CSG_Strings::operator  += (const CSG_String &String)
+bool CSG_Strings::Ins(const CSG_String &String, size_t Index)
 {
-	Add(String);
+	if( Index >= Get_Size() )
+	{
+		return( Add(String) );
+	}
 
-	return( *this );
+	if( m_Strings.Inc_Array() )
+	{
+		for(size_t i=Get_Size()-1; i>Index; i--)
+		{
+			m_Strings[i]	= m_Strings[i - 1];
+		}
+
+		m_Strings[Index]	=  new CSG_String(String);
+
+		return( true );
+	}
+
+	return( false );
 }
 
 //---------------------------------------------------------
-bool CSG_Strings::Set_Count(int nStrings)
+bool CSG_Strings::Del(size_t Index)
 {
-	Clear();
-
-	for(int i=0; i<nStrings; i++)
+	if( Index >= Get_Size() )
 	{
-		Add(SG_T(""));
+		return( false );
+	}
+
+	delete((CSG_String *)m_Strings[Index]);
+
+	for(size_t i=Index+1; i<Get_Size(); i++)
+	{
+		m_Strings[i - 1]	= m_Strings[i];
+	}
+
+	m_Strings.Dec_Array();
+
+	return( true );
+}
+
+//---------------------------------------------------------
+bool CSG_Strings::Set_Count(size_t Count)
+{
+	while( Del(Count) )	{}
+
+	for(size_t i=Get_Size(); i<Count; i++)
+	{
+		Add("");
+	}
+
+	return( true );
+}
+
+//---------------------------------------------------------
+class CSG_Index_Compare_Strings : public CSG_Index::CSG_Index_Compare
+{
+public:
+	CSG_String **m_Values; bool m_Ascending;
+
+	CSG_Index_Compare_Strings(CSG_String **Values, bool Ascending) : m_Values(Values), m_Ascending(Ascending) {}
+
+	virtual int			Compare		(const int _a, const int _b)
+	{
+		int	a	= m_Ascending ? _a : _b;
+		int	b	= m_Ascending ? _b : _a;
+
+		return( m_Values[a]->Cmp(*m_Values[b]) );
+	}
+};
+
+//---------------------------------------------------------
+bool CSG_Strings::Sort(bool Ascending)
+{
+	if( Get_Size() < 2 )
+	{
+		return( true );
+	}
+
+	CSG_Index_Compare_Strings	Compare((CSG_String **)m_Strings.Get_Array(), Ascending);
+
+	CSG_Index	Index(Get_Count(), Compare);
+
+	CSG_Array_Pointer	Strings(m_Strings);
+
+	for(size_t i=0; i<Get_Size(); i++)
+	{
+		m_Strings[i]	= Strings[Index[i]];
 	}
 
 	return( true );
