@@ -189,8 +189,8 @@ CSG_Data_Object::CSG_Data_Object(void)
 	m_File_Type			= 0;
 	m_bModified			= true;
 
-	m_NoData_Value		= -99999.0;
-	m_NoData_hiValue	= -99999.0;
+	m_NoData_Value		= -99999.;
+	m_NoData_hiValue	= -99999.;
 
 	m_Max_Samples		= gSG_DataObject_Max_Samples;
 
@@ -228,11 +228,37 @@ bool CSG_Data_Object::Destroy(void)
 //---------------------------------------------------------
 void CSG_Data_Object::Set_Name(const CSG_String &Name)
 {
-	m_Name	= Name.Length() > 0 ? Name.c_str() : _TL("new");
+	if( Name.is_Empty() )
+	{
+		m_Name	= _TL("Data");
+	}
+	else
+	{
+		m_Name	= Name;
+	}
 }
 
 //---------------------------------------------------------
-void CSG_Data_Object::Set_Name(const char *Format, ...)
+void CSG_Data_Object::Fmt_Name(const char *Format, ...)
+{
+	wxString	_s;
+
+	va_list	argptr;
+	
+#ifdef _SAGA_LINUX
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_start(argptr, _Format); _s.PrintfV(_Format, argptr);
+#else
+	va_start(argptr,  Format); _s.PrintfV( Format, argptr);
+#endif
+
+	va_end(argptr);
+
+	Set_Name(CSG_String(&_s));
+}
+
+//---------------------------------------------------------
+void CSG_Data_Object::Fmt_Name(const wchar_t *Format, ...)
 {
 	wxString	_s;
 
@@ -241,50 +267,17 @@ void CSG_Data_Object::Set_Name(const char *Format, ...)
 #ifdef _SAGA_LINUX
 	// workaround as we only use wide characters
 	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
-	_s.PrintfV(_Format, argptr);
+	wxString _Format(Format); _Format.Replace("%s", "%ls");	// workaround as we only use wide characters since wx 2.9.4 so interpret strings as multibyte
+	va_start(argptr, _Format); _s.PrintfV(_Format, argptr);
 #else
-	va_start(argptr, Format);
-	_s.PrintfV(Format, argptr);
+	va_start(argptr,  Format); _s.PrintfV( Format, argptr);
 #endif
 
 	va_end(argptr);
 
 	CSG_String	s(&_s);
 
-	if( !s.is_Empty() )
-	{
-		Set_Name(s);
-	}
-}
-
-//---------------------------------------------------------
-void CSG_Data_Object::Set_Name(const wchar_t *Format, ...)
-{
-	wxString	_s;
-
-	va_list	argptr;
-	
-#ifdef _SAGA_LINUX
-	// workaround as we only use wide characters
-	// since wx 2.9.4 so interpret strings as multibyte
-	wxString	_Format(Format);	_Format.Replace("%s", "%ls");
-	va_start(argptr, _Format);
-	_s.PrintfV(_Format, argptr);
-#else
-	va_start(argptr, Format);
-	_s.PrintfV(Format, argptr);
-#endif
-
-	va_end(argptr);
-
-	CSG_String	s(&_s);
-
-	if( !s.is_Empty() )
-	{
-		Set_Name(s);
-	}
+	Set_Name(CSG_String(&_s));
 }
 
 //---------------------------------------------------------
@@ -324,7 +317,20 @@ void CSG_Data_Object::Set_File_Name(const CSG_String &FileName, bool bNative)
 //---------------------------------------------------------
 const SG_Char * CSG_Data_Object::Get_File_Name(bool bNative)	const
 {
-	return( !bNative || m_File_bNative ? m_FileName.c_str() : SG_T("") );
+	if( bNative && !m_File_bNative )
+	{
+		return( SG_T("") );
+	}
+
+	if( m_pOwner )
+	{
+		if( m_pOwner->Get_ObjectType() == SG_DATAOBJECT_TYPE_Grids )
+		{
+			return( m_pOwner->m_FileName.c_str() );
+		}
+	}
+
+	return( m_FileName.c_str() );
 }
 
 //---------------------------------------------------------
@@ -405,14 +411,11 @@ bool CSG_Data_Object::Set_NoData_Value_Range(double loValue, double hiValue)
 
 	if( loValue != m_NoData_Value || hiValue != m_NoData_hiValue )
 	{
-		if( !Get_Update_Flag() )
-		{
-			Set_Update_Flag();
-		}
-
 		m_NoData_Value		= loValue;
 		m_NoData_hiValue	= hiValue;
 
+		Set_Modified();
+
 		On_NoData_Changed();
 
 		return( true );
@@ -421,6 +424,17 @@ bool CSG_Data_Object::Set_NoData_Value_Range(double loValue, double hiValue)
 	return( false );
 }
 
+//---------------------------------------------------------
+bool CSG_Data_Object::On_NoData_Changed(void)
+{
+	if( !Get_Update_Flag() )
+	{
+		Set_Update_Flag();
+	}
+
+	return( true );
+}
+
 
 ///////////////////////////////////////////////////////////
 //														 //
@@ -429,11 +443,13 @@ bool CSG_Data_Object::Set_NoData_Value_Range(double loValue, double hiValue)
 //---------------------------------------------------------
 bool CSG_Data_Object::Set_Max_Samples(sLong Max_Samples)
 {
-	if( m_Max_Samples != Max_Samples )
+	#define	Min_Samples	100
+
+	if( m_Max_Samples != Max_Samples && Max_Samples >= Min_Samples )
 	{
 		m_Max_Samples	= Max_Samples;
 
-		Set_Update_Flag();
+        On_Update();
 	}
 
 	return( true );
